pro NDACC_ResolDF2016,dzsampling,coef,hfinp ,status,dzcutf,hfout

; This routine computes vertical resolution using the ISSI-Team-agreed standardized defintion based on the cutoff frequency of digitial filters.
; For details, please refer to ISSI Team Report at http://www.issibern.ch/teams/ndacc/ISSI_Team_Report.htm
; IMPORTANT: If several filtering events occur during data processing, this routine must be called each time a filtering event occurs.
;            The values of "hfout" returned during the first call must be used for "hfinp" in the second call, and so on until the last call.
;            The returned vertical resolution "dzcutf" of the last call only needs to be used (no need for those computed during previosu calls)

; File history:
; Created in 2011 by Thierry Leblanc, NASA-Jet Propulsion Laboratory, thierry.leblanc@jpl.nasa.gov
; Revised in June 2015 by T. Leblanc: Introduce dB threshold: now option is either 0.500 (same as before) or -6dB which equals 0.501
; Revised in Nov. 2015 by T. Leblanc: Introduce input parameter "dzsampling" and use it to compute vertical resolution in physical unit (last line of routine)
; Revised in Nov. 2015 by G. Gronoff: Correct bug in early part of routine (replace "1e9" by "1e-9")
; Revised in Nov. 2015 by G. Gronoff: Add 2 code lines to ensure coefficients are not a mix of status=1 and status=2

; Inputs:
;     dzsampling = Vertical sampling resolution of signal/profile being filtered (in physical unit)
;     coef       = A vector of 2*N+1 coefficients containing the values of the filter coefficients used to smooth or differentiate the signal/profile
;                  The coefficients must be either symmetric (c(k)=c(-k) for all k), or anti-symmteric (c(k)=-c(-k) for all k)
;     hfinp      = A vector (suggested length = total length of your signal/profile) containing the values of the transfer function in the frequency domain.
;                  If routine first call, it is equal to 1 for all frequencies
;                  If not first call, it must be equal to the transfer function "hfout" returned by the routine during the previous call
; Outputs:
;     status = Informaiton flag on computaiton status: 1 = succes (smoothing) , 2 = sucess (differentiation) , 0 = failure (cancel)
;     dzcutf = The value of vertical resolution using the standardized defintion based on the FWHM of an impulse response
;              (same unit as the input sampling resolution)
;     hfout  = A vector of same lenght as hfinp containing the values of the calculated transfer function used to compute the cutoff frequency.
;              If this routine is called again later, use this output as the input transfer function "hfinp" for the next call

ncoef=n_elements(coef) & nf=n_elements(hfinp)
f = 0.5/nf + 0.5/nf*findgen(nf)
hf=dblarr(nf)
missval = -99. & status = 0
hf(*) = missval & hfout = hf
dzcutf = missval & fc = missval & fcinp = missval

; For use later in this routine: Identify cut-off frequency of input Transfer Function, i.e., hfinp(fcinp)=0.5
for jf=0,nf-1-1 do begin
  if ( hfinp(jf+1)-0.5 )*( hfinp(jf)-0.5 ) le 0 then begin
    if abs(hfinp(jf)-0.5) lt 1e-9 then begin
      fcinp=f(jf)
    endif else if abs(hfinp(jf+1)-0.5) lt 1e-9 then begin
      fcinp=f(jf+1)
    endif else begin
      weight=abs(hfinp(jf)-0.5)/abs(hfinp(jf+1)-hfinp(jf))  ; Linear Interpolation between jf and jf+1
      fcinp=f(jf)+weight*abs(f(jf+1)-f(jf))        ; weight = weighted distance between jf and jf+1
    endelse
    break
  endif
endfor

if total(hfinp) eq nf and variance(hfinp) eq 0. then fcinp = 0.5  ; If first call to this routine (i.e., Hfinp(*)=1) => force fcinp to Nyquist frequency (minimum resolution resolved by lidar system)

if ncoef mod 2 eq 0 then begin    ; number of coef is even => reject filter
  status = 0 & return
endif

if ncoef eq 1 and total(coef) ne 1 then begin   ; Only one coef, and this coef is not equal to 1 => reject filter
  status = 0 & print, 'Only one coef, and this coef is not equal to 1 => reject filter' & dzcutf = 0. & hfout=hfinp & return
endif else if ncoef eq 1 and total(coef) eq 1 then begin   ; Only one coef, and this coef = 1 (equivalent to no filtering)
  hf(*) = 1.   ; Transfer Function Hf = 1 at all frequencies
  status = 1 & print, 'Only one coef, and this coef is  equal to 1 => signal unchanged' & dzcutf = dzsampling & hfout=hfinp & return
endif else begin     ; At least 3 coefs, and number of coefs is odd => Calculate Transfer Function Hf (relation ncoef = 2*nc + 1 must hold)
  nc=(ncoef-1)/2
  for ic=1,nc do begin                                                    ; WARNING: with IDL, subscripts start at 0; central point subscript is nc
    if abs(coef(nc-ic)-coef(nc+ic)) le abs(coef(nc-ic))*1e-3 then begin   ;     (with FORTRAN, subscripts start at 1, central point subscript is nc+1)
      if status eq 2 then begin status = 0 & return & endif   ; Coefficients have both status 1 or 2 in the same set => reject
      status = 1                                              ; coef(nc-ic) = coef(nc+ic) (symmetric) =>  Smoothing filter (status=1)
    endif else if (abs(coef(nc-ic)+coef(nc+ic)) le abs(coef(nc-ic))*1e-3) and (coef(nc) eq 0.) then begin
      if status eq 1 then begin status = 0 & return & endif   ; Coefficients have both status 1 or 2 in the same set => reject
      status = 2                                              ; coef(nc-ic) = - coef(nc+ic) and coef(nc)=0 (anti-symmetric) =>  Derivative filter (status=2)
    endif else begin
      status = 0 & return  ; Neither symmetric or anti-symmetric => reject filter
    endelse
  endfor
endelse

; Apply filter (calculate Transfer Function Hf)
hfcomp=complexarr(nf)
for jf=0,nf-1 do begin
  sum=complex([0],[0]) & for ic=-nc,nc do sum = sum + coef(nc-ic)*complex( cos(2*!pi*f(jf)*ic), -sin(2*!pi*f(jf)*ic) )
  hfcomp(jf)=sum
endfor
hf = hfcomp

; Update Transfer Function in case of mutliple calls to NDACC_ResolDF.pro
hfout = hf * hfinp
if status eq 2 then hfout = imaginary(hfout)/(2*!pi*f) else hfout = real_part(hfout)

; Identify cut-off frequency, i.e., find fc where hfout(fc) = 0.501 (-6 dB)
hfc=0.5                    ; If not considering dB, and instead rounding to 0.500
;db=-6. & hfc=10.^(db/20.)  ; If  considering dB, i.e., taking -6 dB = 0.501
for jf=0,nf-1-1 do begin
  if ( hfout(jf+1)-hfc )*( hfout(jf)-hfc ) le 0 then begin
    if abs(hfout(jf)-hfc) lt 1e-9 then begin
      fc=f(jf)
    endif else if abs(hfout(jf+1)-hfc) lt 1e-9 then begin
      fc=f(jf+1)
    endif else begin
      weight=abs(hfout(jf)-hfc)/abs(hfout(jf+1)-hfout(jf))  ; Linear Interpolation between jf and jf+1
      fc=f(jf)+weight*abs(f(jf+1)-f(jf))        ; weight = weighted distance between jf and jf+1
    endelse
    break
  endif
endfor

; If only one coef, and this coef=1 => force cut-off frequency to that of the input Transfer Function hfinp
if ncoef eq 1 then fc = fcinp

; Reverse to spatial domain and convert bin width to resolution (in same unit as dzsampling)
if fc gt 0 then dzcutf = dzsampling/(2.*fc)

return
end