pro NDACC_ResolIR2016,dzsampling,coef,irinp ,status,dzfwhm,irout

; This routine computes vertical resolution using the ISSI-Team-agreed standardized defintion based on the FWHM of an Impulse Response.
; For details, please refer to ISSI Team Report at http://www.issibern.ch/teams/ndacc/ISSI_Team_Report.htm
; IMPORTANT: If several filtering events occur during data processing, this routine must be called each time a filtering event occurs.
;            The values of "irout" returned during the first call must be used for "irinp" in the second call, and so on until the last call.
;            The returned vertical resolution "dzcutf" of the last call only needs to be used (no need for vertical resolution computed during previous calls)

; File history:
; Created in 2011 by Thierry Leblanc, NASA-Jet Propulsion Laboratory, thierry.leblanc@jpl.nasa.gov
; Revised in June 2015 by T. Leblanc: Introduce dB threshold: now option is either 0.500 (same as before) or -6dB which equals 0.501
; Revised in Nov. 2015 by T. Leblanc: Introduce input parameter "dzsampling" and use it to compute vertical resolution in physical unit (last line of routine)
; Revised in Nov. 2015 by G. Gronoff: Add 2 code lines to ensure coefficients are not a mix of status=1 and status=2

; Inputs:
;     dzsampling = Vertical sampling resolution of signal/profile being filtered (in physical unit)
;     coef       = A vector of 2*N+1 coefficients containing the values of the filter coefficients used to smooth or differentiate the signal/profile
;                  The coefficients must be either symmetric (c(k)=c(-k) for all k), or anti-symmteric (c(k)=-c(-k) for all k)
;     irinp      = A vector (suggested length = total length of your signal/profile) containing the values of the impulse.
;                  If routine first call, it is equal to a Kronecker delta function for smoothing filters, and a Heavyside step function for differentiators
;                  If not first call, it must be equal to the impulse response "irout" returned by the routine during the previous call
; Outputs:
;     status = Informaiton flag on computaiton status: 1 = succes (smoothing) , 2 = sucess (differentiation) , 0 = failure (cancel)
;     dzfwhm = The value of vertical resolution using the standardized defintion based on the FWHM of an impulse response
;              (same unit as the input sampling resolution)
;     irout  = A vector of same lenght as hfinp containing the values of the calculated transfer function used to compute the cutoff frequency.
;              If this routine is called again later, use this output as the input impulse "irinp" for the next call

ncoef=n_elements(coef) & nk=n_elements(irinp)
ir=dblarr(nk)
missval = -99. & status = 0
ir(*) = missval & irout = ir
dzfwhm = missval

if ncoef mod 2 eq 0 then begin    ; number of coef is even => reject filter
  status = 0 & return
endif

if ncoef eq 1 and total(coef) ne 1 then begin   ; Only one coef, and this coef is not equal to 1 => reject filter
  status = 0 & return
endif else if ncoef eq 1 and total(coef) eq 1 then begin   ; Only one coef, and this coef = 1 (equivalent to no filtering)
  status = 1
endif else begin     ; At least 3 coefs, and number of coefs is odd => Calculate Transfer Function Hf (relation ncoef = 2*nc + 1 must hold)
  nc=(ncoef-1)/2
  for ic=1,nc do begin                                                    ; WARNING: with IDL, subscripts start at 0; central point subscript is nc
    if abs(coef(nc-ic)-coef(nc+ic)) le abs(coef(nc-ic))*1e-3 then begin   ;     (with FORTRAN, subscripts start at 1, central point subscript is nc+1)
      if status eq 2 then begin status = 0 & return & endif   ; Coefficients have both status 1 or 2 in the same set => reject
      status = 1                                              ; coef(nc-ic) = coef(nc+ic) (symmetric) =>  Smoothing filter (status=1)
    endif else if (abs(coef(nc-ic)+coef(nc+ic)) le abs(coef(nc-ic))*1e-3) and (coef(nc) eq 0.) then begin
      if status eq 1 then begin status = 0 & return & endif   ; Coefficients have both status 1 or 2 in the same set => reject
      status = 2                                              ; coef(nc-ic) = - coef(nc+ic) and coef(nc)=0 (anti-symmetric) =>  Derivative filter (status=2)
     endif else begin
      status = 0 & return  ; Neither symmetric or anti-symmetric => reject filter
    endelse
  endfor
endelse

; If first call to NDACC_ResolIR.pro: Initialize input signal (Dirac at nk/2)
irconvol=irinp & if total(irconvol) eq 0. and stddev(irconvol) eq 0 then irconvol(nk/2)=1.

; If derivative filter: Integrate input signal (Dirac becomes Heavyside step at nk/2)
if status eq 2 then begin
  irtemp=0*irconvol & irtemp(0)=irconvol(0)
  for k=1,nk-1 do irtemp(k) = total(irconvol(0:k)) & irtemp=irtemp/max(irtemp)
  irconvol=irtemp
endif

; Apply filter (calculate Impulse Response)
ir = convol(irconvol,coef)
if max(ir) le 0. then irout = abs(ir) else irout=ir & maxirout=max(irout) & irout=irout/maxirout

; Identify FWHM , i.e., distance between irout=0.5 (up) and irout=0.5 (down)
irc=0.5                    ; If not considering dB, and instead rounding to 0.500
;db=-6. & irc=10.^(db/20.)  ; If  considering dB, i.e., taking -6 dB = 0.501
for k=0,nk-1-1 do begin   ; First on the way up...
  if ( irout(k+1)-irc )*( irout(k)-irc ) le 0 then begin
    if abs(irout(k)-irc) lt 1e-9 then begin
      k1=k & x1=k1
    endif else if abs(irout(k+1)-irc) lt 1e-9 then begin
      k1=k+1 & x1=k1
    endif else begin
      weight=abs(irout(k)-irc)/abs(irout(k+1)-irout(k))  ; Linear Interpolation between k and k+1
      k1=k & x1=k+weight                                 ; weight = weighted distance between k and k+1
    endelse
    break
  endif
endfor

for k=k1+1,nk-1-1 do begin  ; ...then on the way down
  if ( irout(k+1)-irc )*( irout(k)-irc ) le 0 then begin
    if abs(irout(k+1)-irc) lt 1e-9 then begin
      x2=k+1
    endif else if abs(irout(k)-irc) lt 1e-9 then begin
      x2=k
    endif else begin
      weight=abs(irout(k)-irc)/abs(irout(k+1)-irout(k))  ; Linear Interpolation between k and k+1
      x2=k+weight                                        ; weight = weighted distance between k and k+1
    endelse
    break
  endif
endfor

; Convert bin width to resolution (in same units as dzsampling)
dzfwhm = dzsampling*(x2-x1)
irout=irout*maxirout

return
end